/*
* Oupsd:       monitoring des lignes dtr ou rts sur port serie
*              pour simuler un onduleur et arreter une machine
*              sous linux, ayant des problemes de clavier et
*              aucune connection reseau pour se connecter.
* Usage:       oupsd </dev/cua4 (or any other serial device)> [param].
*
* Author:      R. Suinot, <rsuinux@gmx.fr>.
*
* Tres largement inspire de:
*               powerd de Miquel van Smoorenburg, <miquels@drinkel.cistron.nl>
*               en sa version 1.31, du 29-Feb-1996
* et de:
*              setserialbits.c du Alessandro Rubini, rubini@ipvvis.unipv.it.
*
* Version:     0.3  08-Oct-2001
*
*              This program is free software; you can redistribute it and/or
*              modify it under the terms of the GNU General Public License
*              as published by the Free Software Foundation; either version
*              2 of the License, or (at your option) any later version.
*/

/* Use the new way of communicating with init. */
#define NEWINIT
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/ioctl.h>
#include <fcntl.h>
#include <errno.h>
#include <stdlib.h>
#include <unistd.h>
#include <stdio.h>
#include <signal.h>
#include <string.h>
#include "paths.h"
#ifdef NEWINIT
#include "initreq.h"
#endif

#ifndef SIGPWR
# define SIGPWR SIGUSR1
#endif

#ifdef NEWINIT
void alrm_handler()
{
}
#endif

 /* Main program. */
int main(int argc, char **argv)
{
  char *me=argv[0];
  int fd;
  int flags;
  int status;
  uint bits, mask=~0, outb=0;

  if (argc<2 || !strcmp(argv[1],"--help") || !strcmp(argv[1],"-h"))
    {
    printf("%s: Usage: \"%s <device> [options]\"\n",me,me);
    printf("    <device>        is a serial device, like /dev/cua0\n"
           "    \"-rts\" \"+rts\"   clear/set Requst To Send\n"
           "    \"-dtr\" \"+dtr\"   clear/set Data Terminal Ready\n");
    exit(1);
    }

  /* si on ne peut pas ouvrir le periferique de sortie, on quitte */
  if ((fd=open(argv[1], O_RDWR | O_NDELAY)) < 0) {
    printf("%s: %s: %s\n",me,argv[1], sys_errlist[errno]);
    exit(1);
  }

  /* Modif perso */
    while(++argv,--argc)
    {
      if (!strcmp(*argv,"-rts") || !strcmp(*argv,"+rts"))
      {
        mask &= ~TIOCM_RTS;
        outb |= (**argv=='+' ? TIOCM_RTS : 0);
      }
      if (!strcmp(*argv,"-dtr") || !strcmp(*argv,"+dtr"))
      {
        mask &= ~TIOCM_DTR;
        outb |= (**argv=='+' ? TIOCM_DTR : 0);
      }
    }
  printf("Status en cours d'evaluation....\n");
  /* On ouvre la ligne avec les parametres de depart. Force it anyway to be sure. */
  ioctl(fd,TIOCMGET,&bits);
  bits&=mask;
  bits|=outb;
  ioctl(fd,TIOCMSET,&bits);

  printf("\a");
  sleep(3);
  printf("\a\a");
  /* Get the status. */
  ioctl(fd, TIOCMGET, &flags);
  /* Calculate present status. */
  status = (flags & TIOCM_CD);

  printf("Status: %d\n",status);
  return(0);
}
